#!/usr/bin/python3
# -*- coding:utf-8 -*-

import aptdaemon.client
from dbus.mainloop.glib import DBusGMainLoop

import dbus
#import gobject
import logging
from gi.repository import GObject

#logging.basicConfig(level=logging.DEBUG)

def  get_dbus_bus():
   return dbus.SystemBus() 

class AptdaemonTransactionsWatcher:
   """
   base class for objects that need to watch the aptdaemon
   for transaction changes. it registers a handler for the daemon
   going away and reconnects when it appears again
   """
 
   def __init__(self):
      # watch the daemon exit and (re)register the signal
      bus = get_dbus_bus()
      
   def get_active_transactions(self):
      bus = get_dbus_bus()
      apt_daemon = aptdaemon.client.get_aptdaemon(bus=bus)
      current, queued = apt_daemon.GetActiveTransactions()
      if current:
         trans = self.get_transaction(current)
         if trans.status == 'status-committing':
            return 1
         else:
            return 0
      else: 
         logging.debug( "No hay transacciones")
         return 0
    
     
   def register_active_transaction_watch(self):
      self._owner_watcher = bus.watch_name_owner(
             "org.debian.apt", self._register_active_transactions_watch)
   
   #
   #
   #
   def _register_active_transactions_watch(self, connection):
      #print "_register_active_transactions_watch", connection
      bus = get_dbus_bus()
      apt_daemon = aptdaemon.client.get_aptdaemon(bus=bus)
      apt_daemon.connect_to_signal("ActiveTransactionsChanged",
                                   self._on_transactions_changed)
      current, queued = apt_daemon.GetActiveTransactions()
      self._on_transactions_changed(current, queued)
      
      
   def _on_transactions_changed(self, current, queued):
      if not current:
         logging.debug("Hay cero transacciones activas")
      else:
         logging.debug("La transacción activa es: %s" %current)
         logging.debug("Hay %s encoladas y son %s"    %( len(queued), queued))

   def get_transaction(self, tid):
      """ synchroneously return a transaction """
      try:
          trans = aptdaemon.client.get_transaction(tid)
          return trans
      except dbus.DBusException:
          return None

def handler_transaction(current, queued):
   if current:
      print("Hay %s transacciones activas" % current)
      print("Hay %s encoladas y son %s" % (len(queued), queued))
   else:
      print("No hay transacciones activas.")


def main():
   loop = GObject.MainLoop()
   bus = dbus.SystemBus(mainloop=DBusGMainLoop())
   
   apt_client = aptdaemon.client.get_aptdaemon(bus = bus)
   apt_client.connect_to_signal('ActiveTransactionsChanged', 
                                handler_transaction)
   
   current, queued = apt_client.GetActiveTransactions()
   handler_transaction(current, 
                       queued)
   
   loop.run()
   
if __name__ == "__main__":
   watcher = AptdaemonTransactionsWatcher()
   transactions = watcher.get_active_transactions()
   print(transactions)
