#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
#import gobject
#import gtk
#import appindicator
from gi.repository import AppIndicator3 as AppIndicator
from gi.repository import GObject, Gtk
import logging
import time
import os

ICON_GTG_PANEL="gtg-panel.svg"
ICON_APPORT="apport.svg" 
ICON_OK="ok_icon.png"
ICON_WARNING="warning_icon.png"
ICON_UPDATE="update_icon.png"
ICON_ERROR="error_icon.png"

FINISHED_ICON_ARRAY=[]
UPDATING_ICON_ARRAY=[]
# se hace un bucle desde 0 a 8 (EN EL 8 NO ENTRA!)
for i in range(0,11):
    UPDATING_ICON_ARRAY.append('progreso-%s.svg' %str(i))


## Esta clase será la encargada de manejar el icono estático a mostrar en el 
# área de notificaciones de la barra de Gnome. 
# 
# Se implementará utilizando gtk.StatusIcon().
# Funciones interesantes:
#  set_tooltip(str): establece el mensaje del icono
#  set_visible(bool): establece la visibilidad de status_icon
#  set_from_file(str): establece el icono desde un archivo
#   
class Status_icon(object):

   def __init__( self, icons_path):
      # variable definida anteriormente para crear el array
      self.__icons_path = icons_path
      if not os.path.isdir(self.__icons_path):
         raise NameError("No se encuentra la ruta de los iconos")
      self.__menu = Gtk.Menu()
      #self.__menu.show()
      self.__menu_item = Gtk.MenuItem(" ")
      self.__menu.append(self.__menu_item)
      self.__menu_item.show()
      
      self.__menu_resume = Gtk.MenuItem(" ")
      self.__menu.append(self.__menu_resume)

      self.__image = Gtk.MenuItem("Salir")
      self.__image.connect("activate", self.quit)
      self.__menu.append(self.__image)

      t = int(round(time.time() * 1000))
      icon = '%sprogreso-0.svg' % self.__icons_path
      self.__notification = AppIndicator.Indicator.new("cga-update-manager-%s" % t, icon, AppIndicator.IndicatorCategory.SYSTEM_SERVICES)
#      self.__notification = AppIndicator.Indicator.new("cga-update-manager-%s" %(t),
#                                                     "nombre_no_correcto",
#                                                     icon,
#                                                     AppIndicator.IndicatorCategory.SYSTEM_SERVICES)
#                                                     self.__icons_path)
      self.__notification.set_menu(self.__menu)

   
   ## Método que muetra en el menú la opción salir.
   #
   def show_exit_entry(self):
      self.__image.show()


   ## Método que oculta en el menú la opción salir.
   #
   def hide_exit_entry(self):
      self.__image.hide()
      
   ## Método que muetra en el menú la opción lista de paquetes actualizados.
   #   
   def show_resume_entry(self, boolean):
      if boolean == False:
         self.__menu_resume.set_label("Resumen de paquetes instalados")
      else:
         self.__menu_resume.set_label("Resumen del proceso de actualización")
      self.__menu_resume.show()
   
   
   ## Método que cierra la aplicación.
   #
   def quit(self, widget, data=None):
      Gtk.main_quit()
      
   
   ## Método que activa o desactiva el menú la opción lista de paquetes actualizados.
   # @param boolean: Si es True lo activa, si es False lo desactiva.   
   def set_sensitive(self, boolean):
      self.__menu_resume.set_sensitive(boolean)
      
      
   ## Señal que conecta el manejador.
   #
   def set_on_packages_info_clic_handler(self, handler):
      self.__menu_resume.connect("activate", handler)
      
   
   ## @brief muestra el appindicator area de notificaciones de 
   # la barra superior del sistema. Si el icono no existe aparece roto
   #
   def show_icon(self):
      self.__notification.set_status(AppIndicator.IndicatorStatus.ACTIVE)
   
   
   ## Oculta el icono de notificación.
   #
   def hide_icon(self):
      self.__notification.set_status(appindicator.STATUS_PASSIVE)
   
   
   ## @brief Devuelve el estado de visibilidad del icono de notificación.
   # @return True si el icono es visible, False en caso contrario.
   def get_vissible_icon_state(self):
      if ( self.__notification.get_status() == appindicator.STATUS_ACTIVE ):
         return True
      else:
         return False


   ## Cambia el icono del appindicator. En el caso de que el archivo no exista 
   # se callará y el icono aparecerá "roto"
   # 
   # Busca el icono a mostrar en el path indicado en self.__icon_path
   def set_icon(self, name_icon):
      image = "%s%s" % (self.__icons_path, name_icon)
      self.__notification.set_icon(image)

      
   ## @brief Devuelve la ruta al fichero especificada en el método set_icon o None
   # #FIXME: esta función de appindicator no funciona correctamente
   def get_icon(self):
      return self.__notification.get_icon()


   ## Establece el texto como mensaje que saldra al pasar el ratón
   # sobre el icono de notificación.
   # <B>OJO</B>: si el icono no está visible lanza una excepción TODO: ¿que execpción?
   #
   # @param message: cadena del mensaje a mostrar.
   # 
   def set_message(self, message):
      self.__menu_item.set_label(str(message))
      logging.debug( "Establecido mensaje de notificación emergente.")

   
   ## Devuelve el mensaje que se ha establecido para la notificación
   #   emergente o None si no se ha establecido.
   def get_message(self):
      message = self.__menu_item.get_label()
      if message: return message
      else: return None 


## Status_icon animado.
# Se implementará con una array de imágenes que irán refrescandose con
# una frecuencia determinada. 
class Animated_system_icon(Status_icon):
   
   def __init__(self, path):
      super(Animated_system_icon,self).__init__(path)
      # milisegundos entre actualizaciones
      self.__rate = 50
      # identificador del temporizador que controla la actualización de iconos
      self.__timeout_id = None
      # array de imagenes
      self.__images_array = None
      # índice de elemento que estamos recorriendo
      self.__image_index = 0
      
      
   
   ## Establece la array que contiene las imagenes a mostrar.
   # Esta función es la encargada de comprobar que las rutas de las imagenes
   # son correctas
   #
   # @param path_array: contiene cadenas con las rutas absolutas de las imágenes
   # que se iran refrescando
   def set_animated_images_array(self, path_array):
      if (isinstance(path_array, list)):
         self.__images_array = path_array
         logging.debug( "Establecido array de imágenes.")
      else:
         logging.debug( "No se ha especificado un array válido.")
         raise Exception("No se ha especificado un array válido.")

      
   ## Establece la frecuencia de refresco entre las imagenes. El valor
   # por defecto de este parámetro en el constructor es de 500 
   #
   # @param rate: frecuencia en milisegundos de refresco. Por ejemplo: rate=20 
   # refrescará cada 0.2 segundos. 
   def set_refresh_rate(self, rate):
      if (isinstance(rate,int)):
         self.__rate=rate
      else:
         logging.debug( "Debe especificar un valor numérico de tipo integer.")
         raise Exception("No se ha especificado un valor númerico válido.")

   
   ## Devuelve la frecuencia de refresco actual.
   #
   # @return frecuencia de refresco de las imagenes
   def get_refresh_rate(self):
      logging.debug( "Solicitando tasa de refresco de animación.")
      return self.__rate

   
   ## Esta función lanza es la encargada de comenzar la animación de los iconos.
   #
   # Establece el valor del atributo privado con el identificador del temporizador
   # que nos servirá para parar la animación.
   #
   # Funciones relacionadas:
   #   <DL>
   #   <EM>gobject.timeout_add( frec, function)</EM>: temporizador que 
   #   ejecutará cada frec milisegundos la función function. Devuelve un 
   #   identificador del temporizador para así poder pararlo.
   #   </DL>
   #
   # @return True si se ha iniciado la animación. False en otro caso, que puede
   # producirse porque ya se esté ejecutando una animación (identificador de 
   # temporizador diferente de None. 
   def start_animation(self):
      self.__image_index=0
      self.__timeout_id = GObject.timeout_add(self.__rate, self.__next_image)
      logging.debug( "Inicializada la animación del icono.")


   ## Esta función privada es la encargada de ir recorriendo las rutas en  
   # el array de imágenes y establecer la imagen que corresponda.  
   # 
   # Va haciendo un recorrido circular del array con una expresión tipo:
   #    next_index = ( index + 1 ) % len(array)
   def __next_image(self):
      if ( self.__image_index <= len(self.__images_array)-1 ):
         self.set_icon(self.__images_array[self.__image_index])
         #self.update_icon()
         self.__image_index=self.__image_index+1
         if(self.__image_index==len(self.__images_array)):
            self.__image_index=0
      return True
 

   ## Para la animación. Para ello utilizar el atributo privado que contiene el
   # identificador de la animación, establecido en start_animation.
   # 
   # Esta función cuando para la animación poner el atributo identificador del
   # temporizador a None.
   #      
   #   gobject.source_remove(ident) Borra el temporizador que corresponde
   #   con ident.  Devuelve True si ha sido borrado. False en otro caso.
   def stop_animation(self):
      #self.__timeout_id(GObject.source_remove(self.__timeout_id))
      GObject.source_remove(self.__timeout_id)
      logging.debug( "Enviada señal para detener la animación del icono.")
      if ( self.__timeout_id != None ):
         logging.debug( "El gobjecttimer de la animación ha sido eliminado.")
         return True
      else:
         logging.debug( "El gobjecttimer de la animación NO ha sido eliminado.")
         return False

      
def main():
   # para pruebas introducir rutas
   path = "/usr/share/cga-update-manager/icons/"

#   Pruebas sobre método set_icon
   icon = Status_icon("/usr/share/pixmaps")
   icon.set_message("Texto de prueba en hint de icono")
   icon.set_icon("/usr/share/pixmaps/hotot.png")
   icon.show_icon()
   logging.debug("Se muestra el primer icono")
   
#   logging.debug("Se crea un icono animado")
#   icon = Animated_system_icon(path)
#   icon.set_refresh_rate(125)
#   icon.set_animated_images_array(UPDATING_ICON_ARRAY)
#   icon.show_icon()
#   icon.start_animation()
#   icon.set_message("Texto de prueba en hint de icono")
   Gtk.main()
   return 0

if __name__ == "__main__":
   logging.basicConfig(level=logging.DEBUG)
   main()
