#!/usr/bin/python3
#-*- coding: utf-8 -*-

#import pynotify 
import notify2 
from os.path import join as join_path
from os.path import exists as exists_path
from os import getcwd
#import gobject
from gi.repository import GObject
import logging
logging.basicConfig(level=logging.DEBUG)


## Clase que representa una notificación.
#
class Notification:
   ## Constructor de la clase
   # @param tittle: titulo de la notificación. Aparece como una cabecera
   # en el splash de la misma. 
   # @param message: cadena con el mensaje de la notificación
   # @param icon: path absoluto del icono que se mostrará
   # @param life_spam: tiempo de vida de la notificación. El valor por defecto es 4
   def __init__(self, tittle, message, path, icon, life_spam=4):

      self.path = path
      self.icon_path = path + icon
      
      
      if (message == None):
         raise Exception("message argument can't be None")
      self.message    = message
      if (not tittle):
         raise Exception("tittle argument can't be None")
      self.tittle    = tittle
      
      self.life_spam = life_spam
   

## Clase que utilizará pynotify para mostrar los splash de información.
# Es un fachade del módulo original para facilitar su uso.
#
class Notifier:
   
   def __init__(self):
      pass

   ## Función encargada de mostrar la notificación. 
   # Los demás parámetros utilizados por pynotify.Notification los coge 
   # de los atributos del objeto.
   # @param notification: objeto de la clase propia Notification que contiene
   # la información acerca de la notificación
   # @param urgency: int que representa la urgencia. Valores de acuerdo con
   # pynotify.Urgency:
   #   0 LOW
   #   1 NORMAL
   #   2 CRITICAL
   # @return: True si ha conseguido lanzar la notificación. False e.o.c.
   def show_notification( self, notification, urgency=0):
      
      if (urgency < 0 or urgency > 2):
         raise Exception( "Argumento de urgencia inválido: %d . Debe ser un"+ 
                          " valor entero entre 0 y 2" 
                          %(urgency) )
      #else:
      #   urgency=notify2.Urgency(urgency)
         
      if (not notification or
          not isinstance(notification, Notification)  ):
         raise Exception ("notification is an invalid argument: %s" %notification)
      
         
      if not notify2.init("tittle"):
         logging.error( "Imposible de inicializar el notificador" )      
         raise Exception("Error initializing python Notify")
      n = notify2.Notification( notification.tittle, 
                                 notification.message, 
                                 notification.icon_path)
      n.set_urgency(urgency)
      
      if not n.show():
         logging.error( "Error lanzando show_notification de Notification")
         return False
      return True
      
       
if __name__ == "__main__":
   from icon import ICON_UPDATE,ICON_ERROR
   logging.debug( "Probando la clase notification")
   main = GObject.MainLoop()
   
   notifier = Notifier()
   
   n1 = Notification( tittle="Actualización del sistema", 
                      message="Iniciando el proceso", 
                      icon=ICON_UPDATE) 
   
   result = notifier.show_notification( n1)
   if not result:
      pass
      logging.debug( "No se lanza notification")
   
   n2 = Notification( tittle="Actualización del sistema", 
                      message="Error en el proceso", 
                      icon= ICON_ERROR)
   result2 = notifier.show_notification(n2)
   if not result2:
      pass
      logging.error( "No se lanza notifiaction")

