#!/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import dbus
from os.path import isfile
from os import remove
class System_handler:
   
   def __init__(self):
      self.bus = dbus.SystemBus()
      
      self.o_bateria='org.freedesktop.UPower' # Ruta al objeto batería
      self.i_bateria='/org/freedesktop/UPower' # Ruta a la interfaz batería
      
      self.o_red='org.freedesktop.NetworkManager' # Ruta al objeto red   
      self.i_red='/org/freedesktop/NetworkManager' # Ruta a la interfaz red
   
   ## \brief Comprueba si el equipo se encuentra alimentado por batería.
   #
   # \return True si el equipo está siendo alimentado por batería, 
   # False si está conectado a la red eléctrica.
   def battery_available(self):
      check=self.bus.get_object(self.o_bateria,self.i_bateria)
      interface=dbus.Interface(check, 
                               dbus_interface=
                               'org.freedesktop.DBus.Properties')
      if interface.Get(self.o_bateria,'OnBattery')==True:
         return True
         logging.debug( "El sistema está funcionando con la batería.")
      else:
         return False
         logging.debug( "El sistema no está funcionando con la batería.")

   ## \brief Comprueba el estado de la batería. El valor devuelto en el caso 
   # de que el equipo no esté alimentado por  batería es indefinido.
   #
   # @return:  float del porcentaje de la bateria entre 1 (llena) y 0 (vacia) 
   def get_charge_percent(self):
      bat=self.bus.get_object(self.o_bateria, self.i_bateria)
      list=bat.EnumerateDevices(dbus_interface=self.o_bateria)
      if (len(list))>1:
         maxcharge = 0
         for elem in list:
            bat=self.bus.get_object(self.o_bateria, elem)
            interface=dbus.Interface(bat,
                                  dbus_interface='org.freedesktop.DBus.Properties')
            actcharge = interface.Get('org.freedesktop.UPower.Device','Percentage')
            if actcharge > maxcharge:
               maxcharge = actcharge
         return maxcharge
         logging.debug( "Devolviendo % de carga de batería.")
      else:
         return 0
         logging.debug( "El % de la batería es 0, puede que haya algún problema.")

   ## \brief Comprueba si hay conectividad de red. 
   #
   # @return:  Boolean indicando si hay conectividad (True) o si no hay 
   # conectividad (False)
   def check_connection(self):
      eth = self.bus.get_object(self.o_red, self.i_red)
      manager = dbus.Interface(eth, self.o_red)
      devices = manager.GetDevices()
      estado = False
      for dev in devices:
          dev_proxy = self.bus.get_object(self.o_red, dev)
          prop_iface = dbus.Interface(dev_proxy, "org.freedesktop.DBus.Properties")
          state = prop_iface.Get("org.freedesktop.NetworkManager.Device", "State")
          name = prop_iface.Get("org.freedesktop.NetworkManager.Device", "Interface")

          if state != 20 and state != 10 and name != 'lo':
              estado = True
      return estado

#if __name__ == "__main__":
#   logging.debug( "Ejecutando pruebas de la clase System_handler")
#   S = System_handler()
#
#   
#   print S.check_connection()
